package com.malk.poc.controller;

/**
 * 错误抛出与拦截详见 CatchException
 */

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.malk.poc.server.AWServer;
import com.malk.poc.service.AWClint;
import com.malk.poc.service.AWYDClient;
import com.malk.delegate.McDelegate;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McException;
import com.malk.server.common.McR;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.teambition.TBConf;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import com.malk.service.teambition.TBClient;
import com.malk.utils.UtilExcel;
import com.malk.utils.UtilMap;
import com.malk.utils.UtilServlet;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Slf4j
@RestController
@RequestMapping
public class TBxYDController {

    @Autowired
    private AWClint awClint;

    /**
     * 通过模板创建项目
     */
    @PostMapping("project/create")
    McR createProject(@RequestBody JSONObject data) {

//        Map<String, ?> data = UtilServlet.getParamMap(request);
        log.info("通过模板创建项目, {}", data);
        McException.assertParamException_Null(data, "projectCode");
        String templateId = UtilMap.getString(data, "templateId");
        String user = UtilMap.getString(data, "user");
        if (data.containsKey("manual")) {
            awClint.createProject(UtilMap.getString(data, "projectCode"), templateId,data.getString("tbProjectId"),user);
        } else {
            // ppExt: 宜搭数据创建后, 以及客户系统同步到主数据有时间差, 做延迟代理 10s
            mcDelegate.setTimeout(() -> {
                awClint.createProject(UtilMap.getString(data, "projectCode"), templateId,data.getString("tbProjectId"),user);
            }, 10000);
        }
        return McR.success();
    }

    /**
     * 分配项目角色
     */
    @PostMapping("project/role")
    McR updateRoleProject(@RequestBody JSONObject data) {

//        Map<String, ?> data = UtilServlet.getParamMap(request);
        log.info("分配项目角色, {}", data);
        McException.assertParamException_Null(data, "projectId", "trNode");

        List<String> trNodes = data.getJSONArray("trNode").toJavaList(String.class);
        awClint.updateProjectRole(UtilMap.getString(data, "projectId"), trNodes, UtilMap.getString(data, "production"), UtilMap.getString(data, "version"));
        return McR.success();
    }

    /**
     * 创建紧急任务
     */
    @PostMapping("task/create")
    McR createTaskForIpd(@RequestBody JSONObject data) {
        log.info("创建任务, {}", data);
        McException.assertParamException_Null(data, "formInstId");
        awClint.createTask(data.getString("formInstId"));
        return McR.success();
    }

    /**
     * 更新里程碑时间点
     */
    @PostMapping("task/updateLcb")
    McR updateLcbTask(@RequestBody JSONObject data) {
        log.info("更新里程碑时间, {}", data);
        McException.assertParamException_Null(data, "projectCode");
        if (data.containsKey("manual")) {
            awClint.updateLcb(data.getString("projectCode"));
        } else {
            // ppExt: 宜搭数据创建后, 以及客户系统同步到主数据有时间差, 做延迟代理 10s
            mcDelegate.setTimeout(() -> {
                awClint.updateLcb(data.getString("projectCode"));
            }, 10000);
        }
        return McR.success();
    }

    /**
     * 交付物审批
     */
    @PostMapping("do-approve")
    McR doApprove(@RequestBody JSONObject data) {
        log.info("交付物审批, {}", data);
        Map formData = awClint.doApprove(data);
        return McR.success(formData);
    }

    @Autowired
    private TBClient tbClient;
    @Autowired
    private TBConf tbConf;

    @PostMapping("queryTask")
    McR queryTask(@RequestBody JSONObject data) {
        log.info("查询任务信息, {}", data);
        Map formData = tbClient.queryTaskDetail(data.getString("taskId"),"","").get(0);
        return McR.success(formData);
    }

    @PostMapping("updateTaskDueDate")
    McR updateTaskDueDate(@RequestBody JSONObject data) {
        log.info("更新任务截至时间, {}", data);
        Map formData = tbClient.updateTaskDueDate(data.getString("taskId"),data.getString("dueDate"),tbConf.getOperatorId());
        return McR.success(formData);
    }

//    /**
//     * 检查项check回调
//     */
//    @PostMapping("checked")
//    McR check(HttpServletRequest request) {
//
//        Map<String, ?> data = UtilServlet.getParamMap(request);
//        log.info("检查项check回调, {}", data);
//        McException.assertParamException_Null(data, "taskId, formInstId");
//
//        awClint.checked(data);
//        return McR.success();
//    }

    /**
     * 交付物审批回调
     */
    @PostMapping("approved")
    McR approve(@RequestBody JSONObject data) {
        log.info("交付物审批回调, {}", data);
        McException.assertParamException_Null(data, "projectId, taskId, approve");
        awClint.approved(data);
        return McR.success();
    }


    @Autowired
    private McDelegate mcDelegate;

    @GetMapping("tmp")
    McR tmp(String programIds) {
        return McR.success();
    }
}
